@@ -3,11 +3,13 @@ package com.jetbrains.python.psi.stubs;
 
 import com.intellij.psi.stubs.NamedStub;
 import com.intellij.psi.util.QualifiedName;
-import com.jetbrains.python.psi.PyTargetExpression;
+import com.jetbrains.python.psi.*;
+import org.jetbrains.annotations.ApiStatus;
 import org.jetbrains.annotations.Nullable;
 
 
-public interface PyTargetExpressionStub extends NamedStub<PyTargetExpression>, PyTypeCommentOwnerStub, PyAnnotationOwnerStub {
+public interface PyTargetExpressionStub
+  extends NamedStub<PyTargetExpression>, PyTypeCommentOwnerStub, PyAnnotationOwnerStub, PyVersionSpecificStub {
   enum InitializerType {
     ReferenceExpression(1),
     CallExpression(2),
@@ -25,12 +27,12 @@ public interface PyTargetExpressionStub extends NamedStub<PyTargetExpression>, P
     }
 
     public static InitializerType fromIndex(int index) {
-      switch (index) {
-        case 1: return ReferenceExpression;
-        case 2: return CallExpression;
-        case 3: return Custom;
-        default: return Other;
-      }
+      return switch (index) {
+        case 1 -> ReferenceExpression;
+        case 2 -> CallExpression;
+        case 3 -> Custom;
+        default -> Other;
+      };
     }
   }
 
@@ -39,13 +41,13 @@ public interface PyTargetExpressionStub extends NamedStub<PyTargetExpression>, P
   @Nullable
   QualifiedName getInitializer();
 
+  @ApiStatus.Internal
+  @Nullable
+  PyLiteralKind getAssignedLiteralKind();
+
   boolean isQualified();
 
-  /**
-   * @deprecated It is our internal API, try to avoid using it.
-   * It is planned to be removed sooner or later, so please don't rely on this method.
-   */
-  @Deprecated(forRemoval = true)
+  @ApiStatus.Internal
   @Nullable
   <T> T getCustomStub(Class<T> stubClass);
 
