@@ -1,6 +1,8 @@
 // Copyright 2000-2024 JetBrains s.r.o. and contributors. Use of this source code is governed by the Apache 2.0 license.
 package com.jetbrains.python.psi.types;
 
+import com.intellij.openapi.diagnostic.Logger;
+import com.intellij.openapi.progress.ProcessCanceledException;
 import com.intellij.openapi.project.Project;
 import com.intellij.openapi.roots.ProjectFileIndex;
 import com.intellij.openapi.util.Pair;
@@ -17,6 +19,7 @@ import com.jetbrains.python.psi.*;
 import com.jetbrains.python.psi.impl.PyTypeProvider;
 import com.jetbrains.python.psi.resolve.PyResolveContext;
 import com.jetbrains.python.psi.resolve.RatedResolveResult;
+import com.jetbrains.python.pyi.PyiLanguageDialect;
 import org.jetbrains.annotations.ApiStatus;
 import org.jetbrains.annotations.NotNull;
 import org.jetbrains.annotations.Nullable;
@@ -24,7 +27,6 @@ import org.jetbrains.annotations.Nullable;
 import java.util.ArrayList;
 import java.util.List;
 import java.util.Map;
-import java.util.concurrent.ConcurrentHashMap;
 import java.util.function.Function;
 
 
@@ -50,24 +52,15 @@ public sealed class TypeEvalContext {
 
   private final ThreadLocal<ProcessingContext> myProcessingContext = ThreadLocal.withInitial(ProcessingContext::new);
 
-  protected final Map<PyTypedElement, PyType> myEvaluated = createMap();
-  protected final Map<PyCallable, PyType> myEvaluatedReturn = createMap();
-  protected final Map<Pair<PyExpression, Object>, PyType> contextTypeCache = createMap();
-  /**
-   * AssumptionContext invariant requires that if type is in the map, 
-   * it's dependencies are also in the map, so we can't use softValueMap.
-   * Temporary solution until we know assumeType works as expected.
-   * @see TypeEvalContext#assumeType(PyTypedElement, PyType, Function) 
-   */
-  private static <T> Map<T, PyType> createMap() {
-    if (Registry.is("python.use.better.control.flow.type.inference")) {
-      return new ConcurrentHashMap<>();
-    }
-    return CollectionFactory.createConcurrentSoftValueMap();
-  }
+  protected final Map<PyTypedElement, PyType> myEvaluated = CollectionFactory.createConcurrentSoftValueMap();
+  public final Map<PyTypedElement, PyType> myExternalEvaluated = CollectionFactory.createConcurrentSoftValueMap();
+  protected final Map<PyCallable, PyType> myEvaluatedReturn = CollectionFactory.createConcurrentSoftValueMap();
+  protected final Map<Pair<PyExpression, Object>, PyType> contextTypeCache = CollectionFactory.createConcurrentSoftValueMap();
+
+  protected static final Logger logger = Logger.getInstance(TypeEvalContext.class);
 
   private TypeEvalContext(boolean allowDataFlow, boolean allowStubToAST, boolean allowCallContext, @Nullable PsiFile origin) {
-    myConstraints = new TypeEvalConstraints(allowDataFlow, allowStubToAST, allowCallContext, origin);
+    this(new TypeEvalConstraints(allowDataFlow, allowStubToAST, allowCallContext, origin));
   }
 
   private TypeEvalContext(@NotNull TypeEvalConstraints constraints) {
@@ -81,15 +74,19 @@ public sealed class TypeEvalContext {
   }
 
   public boolean allowDataFlow(PsiElement element) {
-    return myConstraints.myAllowDataFlow || inOrigin(element);
+    return myConstraints.myAllowDataFlow && !inPyiFile(element) || inOrigin(element);
   }
 
   public boolean allowReturnTypes(PsiElement element) {
-    return myConstraints.myAllowDataFlow || inOrigin(element);
+    return myConstraints.myAllowDataFlow && !inPyiFile(element) || inOrigin(element);
   }
 
   public boolean allowCallContext(@NotNull PsiElement element) {
-    return myConstraints.myAllowCallContext && inOrigin(element);
+    return myConstraints.myAllowCallContext && !inPyiFile(element) && inOrigin(element);
+  }
+
+  public boolean maySwitchToAST(@NotNull PsiElement element) {
+    return myConstraints.myAllowStubToAST && !inPyiFile(element) || inOrigin(element);
   }
 
   /**
@@ -201,6 +198,9 @@ public sealed class TypeEvalContext {
 
   @ApiStatus.Internal
   public <R> @Nullable R assumeType(@NotNull PyTypedElement element, @Nullable PyType type, @NotNull Function<TypeEvalContext, R> func) {
+    if (!Registry.is("python.use.better.control.flow.type.inference")) {
+      return func.apply(this);
+    }
     if (getKnownType(element) != null) {
       // Temporary solution, as overwriting known type might introduce inconsistencies with its dependencies.
       return null;
@@ -221,6 +221,11 @@ public sealed class TypeEvalContext {
     return this instanceof AssumptionContext;
   }
 
+  @ApiStatus.Internal
+  public boolean isKnown(PyTypedElement element) {
+    return getKnownType(element) != null;
+  }
+
   protected @Nullable PyType getKnownType(final @NotNull PyTypedElement element) {
     if (element instanceof PyInstantTypeProvider) {
       return element.getType(this, Key.INSTANCE);
@@ -230,6 +235,11 @@ public sealed class TypeEvalContext {
       assertValid(cachedType, element);
       return cachedType;
     }
+    final PyType cachedExternalType = myExternalEvaluated.get(element);
+    if (cachedExternalType != null) {
+      assertValid(cachedExternalType, element);
+      return cachedExternalType;
+    }
     return null;
   }
 
@@ -242,6 +252,11 @@ public sealed class TypeEvalContext {
     return null;
   }
 
+  @ApiStatus.Experimental
+  public void putExternalType(PyTypedElement element, PyType type) {
+    myExternalEvaluated.put(element, type == null ? PyNullType.INSTANCE : type);
+  }
+
   private static boolean isLibraryElement(@NotNull PsiElement element) {
     PsiFile containingFile = element.getContainingFile();
     VirtualFile vFile = containingFile == null ? null : containingFile.getOriginalFile().getVirtualFile();
@@ -278,6 +293,24 @@ public sealed class TypeEvalContext {
       Pair.create(element, this),
       false,
       () -> {
+        // Try external providers first
+        for (var provider : TypeEvalExternalTypeProvider.EP_NAME.getExtensionList()) {
+          try {
+            var provided = provider.provideType(element, this);
+            if (provided != null) {
+              var type = provided.get();
+              myExternalEvaluated.put(element, type == null ? PyNullType.INSTANCE : type);
+              return type;
+            }
+          }
+          catch (ProcessCanceledException e) {
+            throw e;
+          }
+          catch (Exception e) {
+            logger.warn("Exception during external type provider " + provider.getClass().getName(), e);
+          }
+        }
+
         PyType type = element.getType(this, Key.INSTANCE);
         assertValid(type, element);
         myEvaluated.put(element, type == null ? PyNullType.INSTANCE : type);
@@ -328,10 +361,6 @@ public sealed class TypeEvalContext {
     }
   }
 
-  public boolean maySwitchToAST(@NotNull PsiElement element) {
-    return myConstraints.myAllowStubToAST || inOrigin(element);
-  }
-
   public @Nullable PsiFile getOrigin() {
     return myConstraints.myOrigin;
   }
@@ -366,7 +395,22 @@ public sealed class TypeEvalContext {
   }
 
   private boolean inOrigin(@NotNull PsiElement element) {
-    return myConstraints.myOrigin == element.getContainingFile() || myConstraints.myOrigin == getContextFile(element);
+    return isSameVirtualFile(myConstraints.myOrigin, element.getContainingFile()) ||
+           isSameVirtualFile(myConstraints.myOrigin, getContextFile(element));
+  }
+
+  private static boolean isSameVirtualFile(@Nullable PsiFile file1, @Nullable PsiFile file2) {
+    if (file1 == null) return false;
+    if (file2 == null) return false;
+    return file1.getViewProvider().getVirtualFile().equals(file2.getViewProvider().getVirtualFile());
+  }
+
+  private static boolean inPyiFile(@NotNull PsiElement element) {
+    if (isPyiFile(element.getContainingFile())) {
+      return true;
+    }
+    PsiFile contextFile = getContextFile(element);
+    return contextFile != null && isPyiFile(contextFile);
   }
 
   private static PsiFile getContextFile(@NotNull PsiElement element) {
@@ -381,6 +425,10 @@ public sealed class TypeEvalContext {
     }
   }
 
+  private static boolean isPyiFile(@NotNull PsiFile file) {
+    return file.getLanguage().equals(PyiLanguageDialect.getInstance());
+  }
+
   private static class PyNullType implements PyType {
     private PyNullType() {}
 
@@ -463,6 +511,18 @@ public sealed class TypeEvalContext {
     }
   }
 
+  final static class LibraryTypeEvalContext extends TypeEvalContext {
+    private LibraryTypeEvalContext(@NotNull TypeEvalConstraints constraints) {
+      super(constraints);
+    }
+
+    @Override
+    protected boolean canDelegateToLibraryContext(PyTypedElement element) {
+      // It's already the library-context.
+      return false;
+    }
+  }
+
   final static class OptimizedTypeEvalContext extends TypeEvalContext {
     private volatile TypeEvalContext codeInsightFallback;
 
@@ -479,7 +539,7 @@ public sealed class TypeEvalContext {
         }
       }
       TypeEvalConstraints constraints = getConstraints();
-      return constraints.myOrigin != null && file != constraints.myOrigin && (file instanceof PyFile) &&
+      return constraints.myOrigin != null && !isSameVirtualFile(file, constraints.myOrigin) && (file instanceof PyFile) &&
              !constraints.myAllowDataFlow && !constraints.myAllowStubToAST && !constraints.myAllowCallContext;
     }
 
@@ -522,16 +582,4 @@ public sealed class TypeEvalContext {
       return super.getReturnType(callable);
     }
   }
-
-  final static class LibraryTypeEvalContext extends TypeEvalContext {
-    private LibraryTypeEvalContext(@NotNull TypeEvalConstraints constraints) {
-      super(constraints);
-    }
-
-    @Override
-    protected boolean canDelegateToLibraryContext(PyTypedElement element) {
-      // It's already the library-context.
-      return false;
-    }
-  }
 }
