@@ -1,11 +1,13 @@
 import os
 import sys
 
+from _pydevd_bundle.pydevd_constants import IS_PY312_OR_GREATER
+
 use_cython = os.getenv('PYDEVD_USE_CYTHON', None)
 
 if use_cython == 'NO':
     from _pydevd_bundle import pydevd_pep_669_tracing as mod
-elif use_cython == 'YES':
+elif use_cython == 'YES' and IS_PY312_OR_GREATER:
     from _pydevd_bundle import pydevd_pep_669_tracing_cython as mod
 else:
     try:
@@ -16,13 +18,14 @@ else:
 sys.modules['_pydevd_bundle.pydevd_pep_669_tracing'] = mod
 
 enable_pep669_monitoring = mod.enable_pep669_monitoring
+disable_pep669_monitoring = mod.disable_pep669_monitoring
 global_cache_skips = mod.global_cache_skips
 global_cache_frame_skips = mod.global_cache_frame_skips
 
+def _dummy_restart_events():
+    pass
+
 try:
     restart_events = sys.monitoring.restart_events
 except AttributeError:
-    def _dummy_restart_events():
-        pass
-
-    restart_events = _dummy_restart_events
\ No newline at end of file
+    restart_events = _dummy_restart_events
