@@ -4,7 +4,6 @@ import ctypes
 import os
 import signal
 import traceback
-
 import pydevd_file_utils
 
 try:
@@ -631,6 +630,12 @@ def should_evaluate_shape():
     return LOAD_VALUES_POLICY != ValuesPolicy.ON_DEMAND
 
 
+def has_attribute_safe(obj, attr_name):
+    """Evaluates the existence of attribute without accessing it."""
+    attr = inspect.getattr_static(obj, attr_name, None)
+    return attr is not None
+
+
 def is_safe_to_access(obj, attr_name):
     """Evaluates the safety of attribute accessibility via `obj.attr_name`.
 
@@ -640,11 +645,21 @@ def is_safe_to_access(obj, attr_name):
     of attribute access in the most risk-free manner. As an example, it leverages the
     `inspect` module, facilitating attribute retrieval without triggering any
     descriptor functionality.
+
+    Note
+    ----
+    This function performs a strict check for potential side-effects, the access can be safe even if `False` is returned.
+    This might need to be checked more precisely for some special types.
     """
+
     attr = inspect.getattr_static(obj, attr_name, None)
 
+    # Filter out objects that don't contain the given attribute
+    if attr is None:
+        return False
+
     # Should we check for other descriptor types here?
-    if inspect.isgetsetdescriptor(attr):
+    if inspect.isgetsetdescriptor(attr) or isinstance(attr, property):
         return False
 
     return True
